/**
 ** Copyright 2010, QNX Software Systems Ltd. All Rights Reserved
 **
 ** Permission to use, copy, modify, and distribute this software and its
 ** documentation for any purpose and without fee is hereby granted,
 ** provided that the above copyright notice appear in all copies and that
 ** both that copyright notice and this permission notice appear in
 ** supporting documentation.
 **
 ** This file is provided AS IS with no warranties of any kind.  The author
 ** shall have no liability with respect to the infringement of copyrights,
 ** trade secrets or any patents by this file or any part thereof.  In no
 ** event will the author be liable for any lost revenue or profits or
 ** other special, indirect and consequential damages.
 */

#include <ctype.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <screen/screen.h>
#include <sys/stat.h>

int main(int argc, char **argv)
{
	screen_context_t screen_ctx;
	screen_display_t screen_disp = NULL;
	screen_display_t *screen_displays;
	screen_pixmap_t screen_pix;
	screen_buffer_t screen_buf;
	char *fname = "screenshot.bmp";
	char *disp = NULL;
	char *tok;
	char header[54];
	int size[2] = { 0, 0 };
	int val;
	void *pointer;
	int stride;
	int nbytes;
	int count, id, type;
	int i, fd, rc;

	for (i = 1; i < argc; i++) {
		if (strncmp(argv[i], "-size=", strlen("-size=")) == 0) {
			tok = argv[i] + strlen("-size=");
			size[0] = atoi(tok);
			while (*tok >= '0' && *tok <= '9') {
				tok++;
			}
			size[1] = atoi(tok+1);
		} else if (strncmp(argv[i], "-display=", strlen("-display=")) == 0) {
			disp = argv[i] + strlen("-display=");
		} else if (strncmp(argv[i], "-file=", strlen("-file=")) == 0) {
			fname = argv[i] + strlen("-file=");
		} else {
			fprintf(stderr, "invalid command line option: %s\n", argv[i]);
		}
	}

	rc = screen_create_context(&screen_ctx, SCREEN_WINDOW_MANAGER_CONTEXT);
	if (rc) {
		perror("screen_context_create");
		return 1;
	}

	count = 0;
	rc = screen_get_context_property_iv(screen_ctx, SCREEN_PROPERTY_DISPLAY_COUNT, &count);
	if (rc) {
		perror("screen_get_context_property_iv(SCREEN_PROPERTY_DISPLAY_COUNT)");
		return 1;
	}

	if (count > 0) {
		screen_displays = calloc(count, sizeof(screen_display_t));
		rc = screen_get_context_property_pv(screen_ctx, SCREEN_PROPERTY_DISPLAYS, (void **)screen_displays);
		if (rc) {
			perror("screen_get_context_property_pv(SCREEN_PROPERTY_DISPLAYS)");
			return 1;
		}

		if (!disp) {
			screen_disp = screen_displays[0];
		} else {
			if (isdigit(*disp)) {
				id = strtoul(disp, 0, NULL);
				for (i = 0; i < count; i++) {
					rc = screen_get_display_property_iv(screen_displays[i], SCREEN_PROPERTY_ID, &val);
					if (val == id) {
						screen_disp = screen_displays[i];
						break;
					}
				}
			} else {
				if (!strcmp(disp, "internal")) {
					type = SCREEN_DISPLAY_TYPE_INTERNAL;
				} else if (!strcmp(disp, "composite")) {
					type = SCREEN_DISPLAY_TYPE_COMPOSITE;
				} else if (!strcmp(disp, "svideo")) {
					type = SCREEN_DISPLAY_TYPE_SVIDEO;
				} else if (!strcmp(disp, "YPbPr")) {
					type = SCREEN_DISPLAY_TYPE_COMPONENT_YPbPr;
				} else if (!strcmp(disp, "rgb")) {
					type = SCREEN_DISPLAY_TYPE_COMPONENT_RGB;
				} else if (!strcmp(disp, "rgbhv")) {
					type = SCREEN_DISPLAY_TYPE_COMPONENT_RGBHV;
				} else if (!strcmp(disp, "dvi")) {
					type = SCREEN_DISPLAY_TYPE_DVI;
				} else if (!strcmp(disp, "hdmi")) {
					type = SCREEN_DISPLAY_TYPE_HDMI;
				} else {
					type = SCREEN_DISPLAY_TYPE_OTHER;
				}
				for (i = 0; i < count; i++) {
					rc = screen_get_display_property_iv(screen_displays[i], SCREEN_PROPERTY_TYPE, &val);
					if (val == type) {
						screen_disp = screen_displays[i];
						break;
					}
				}
			}
		}

		free(screen_displays);
	}

	if (!screen_disp) {
		fputs("no displays\n", stderr);
		return 1;
	}

	rc = screen_create_pixmap(&screen_pix, screen_ctx);
	if (rc) {
		perror("screen_create_pixmap");
		return 1;
	}

	val = SCREEN_USAGE_READ | SCREEN_USAGE_NATIVE;
	rc = screen_set_pixmap_property_iv(screen_pix, SCREEN_PROPERTY_USAGE, &val);
	if (rc) {
		perror("screen_set_pixmap_property_iv(SCREEN_PROPERTY_USAGE)");
		return 1;
	}

	val = SCREEN_FORMAT_RGBA8888;
	rc = screen_set_pixmap_property_iv(screen_pix, SCREEN_PROPERTY_FORMAT, &val);
	if (rc) {
		perror("screen_set_pixmap_property_iv(SCREEN_PROPERTY_FORMAT)");
		return 1;
	}

	if (size[0] <= 0 || size[1] <= 0) {
		rc = screen_get_display_property_iv(screen_disp, SCREEN_PROPERTY_SIZE, size);
		if (rc) {
			perror("screen_get_display_property_iv(SCREEN_PROPERTY_SIZE)");
			return 1;
		}
	}

	rc = screen_set_pixmap_property_iv(screen_pix, SCREEN_PROPERTY_BUFFER_SIZE, size);
	if (rc) {
		perror("screen_set_pixmap_property_iv(SCREEN_PROPERTY_BUFFER_SIZE)");
		return 1;
	}

	rc = screen_create_pixmap_buffer(screen_pix);
	if (rc) {
		perror("screen_create_pixmap_buffer");
		return 1;
	}

	rc = screen_get_pixmap_property_pv(screen_pix, SCREEN_PROPERTY_RENDER_BUFFERS, (void **)&screen_buf);
	if (rc) {
		perror("screen_get_pixmap_property_pv(SCREEN_PROPERTY_RENDER_BUFFERS)");
		return 1;
	}

	rc = screen_get_buffer_property_pv(screen_buf, SCREEN_PROPERTY_POINTER, &pointer);
	if (rc) {
		perror("screen_get_buffer_property_pv(SCREEN_PROPERTY_POINTER)");
		return 1;
	}

	rc = screen_get_buffer_property_iv(screen_buf, SCREEN_PROPERTY_STRIDE, &stride);
	if (rc) {
		perror("screen_get_buffer_property_pv(SCREEN_PROPERTY_STRIDE)");
		return 1;
	}

	rc = screen_read_display(screen_disp, screen_buf, 0, NULL, 0);
	if (rc) {
		perror("screen_read_display");
		return 1;
	}

	fd = open(fname, O_CREAT|O_WRONLY);
	if (fd == -1) {
		perror("open");
		return 1;
	}

	nbytes = size[0] * size[1] * 4;
	header[0] = 'B';
	header[1] = 'M';
	header[2] = nbytes & 0xff;
	header[3] = (nbytes >> 8) & 0xff;
	header[4] = (nbytes >> 16) & 0xff;
	header[5] = (nbytes >> 24) & 0xff;
	header[6] = 0;
	header[7] = 0;
	header[8] = 0;
	header[9] = 0;
	header[10] = 54;
	header[11] = 0;
	header[12] = 0;
	header[13] = 0;
	header[14] = 40;
	header[15] = 0;
	header[16] = 0;
	header[17] = 0;
	header[18] = size[0] & 0xff;
	header[19] = (size[0] >> 8) & 0xff;
	header[20] = (size[0] >> 16) & 0xff;
	header[21] = (size[0] >> 24) & 0xff;
	header[22] = -size[1] & 0xff;
	header[23] = (-size[1] >> 8) & 0xff;
	header[24] = (-size[1] >> 16) & 0xff;
	header[25] = (-size[1] >> 24) & 0xff;
	header[26] = 1;
	header[27] = 0;
	header[28] = 32;
	header[29] = 0;
	header[30] = 0;
	header[31] = 0;
	header[32] = 0;
	header[33] = 0;
	header[34] = 0; /* image size */
	header[35] = 0;
	header[36] = 0;
	header[37] = 0;
	header[38] = 0x9;
	header[39] = 0x88;
	header[40] = 0;
	header[41] = 0;
	header[42] = 0x9l;
	header[43] = 0x88;
	header[44] = 0;
	header[45] = 0;
	header[46] = 0;
	header[47] = 0;
	header[48] = 0;
	header[49] = 0;
	header[50] = 0;
	header[51] = 0;
	header[52] = 0;
	header[53] = 0;
	write(fd, header, sizeof(header));

	for (i = 0; i < size[1]; i++) {
		write(fd, pointer + i * stride, size[0] * 4);
	}

	close(fd);
	screen_destroy_pixmap(screen_pix);
	screen_destroy_context(screen_ctx);

	fd = chmod(fname, S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
		if (fd == -1) {
			perror("chmod");
			return 1;
		}
	return 0;
}

#if defined(__QNXNTO__) && defined(__USESRCVERSION)
#include <sys/srcversion.h>
__SRCVERSION("$URL: http://svn/product/branches/6.6.0/trunk/apps/screen/utils/screenshot/screenshot.c $ $Rev: 711469 $")
#endif
